function paramsREM=initEstes(paramsGen, paramsREM)
%Diese Methode liest die Lerndatei ein und initialisiert die
%Gewichte zwischen der Input- und der Output-Schicht. Vorerst
%wird davon ausgegangen, daß (wie bei Konditionierung üblich)
%nur ein Ausgabeneuron existiert. Die Datei mit den Lernmustern
%kodiert die An- und Abwesenheit der einzelnen CS. Diese Muster
%werden intern entsprechend der angegebenen Elementkonfiguration
%auf einen Eingabevektor abgebildet.


%The generel parameters passed by paramsGen are
% Stimulusname: name of all trial types: cell
% Testname:     "
% Kont:         US lambda for all trial types: noTrialTypes-vector
% KontTest:     "
% beta:         US beta for all trial types: noTrialTypes-vector
% AnzVp:        number of sequences: double
% CS:           CS-sequence: noTrial*AnzVp-matrix
% phaseInfo     phase of current trial:  noTrial-vector 
% US:           US-sequence: noTrial*AnzVp-matrix
% block:        trials trained in each block in each phase: cell matrix for each phase
% code:         CS for each trial types (without unique cue or similar): noTrialTypes*noCS-matrix
% testcode:     "


%paramsREM:         parameter for Replaced Elements Model
% resultsPath:      directory where resulting matrices are saved
% startingWeights:  txt-file with associative weights whose last line is used as starting
% alpha             vector with alpha parameter
% proportionPath    txt-file with replaced proportions
% pattern:          expanded stimulus code
% noWeights:        number of elements needed to represent all stimuli combinations
% rule:             learning rule of Schultheis et al or Glautier?
% weightPlot:       checkbox for weightPlot
% activationPlots:  checkbox for activatiaonPlots

% Variablen
% noCS:             number of single stimuli
% noWeights:        necessary number of elements to represent all components' combinations
% proportion:       TODO

% Copyright (C) 2006-2017  Anna Thorwart and Holger Schultheis

noCS = size(paramsGen.code,2);
noWeights = 1;

%compute param.noWeights
for i = 1:(noCS - 1)
    noWeights = noWeights + binomCoeff(noCS - 1, i) * (2^i);
end
paramsREM.noWeights = noWeights * noCS;


% check alphas
NoAlphas_theo=noCS; %number of alphas nedded
NoAlphas_be=size(paramsREM.alpha,2); %number of alphas entered

if NoAlphas_theo ~=NoAlphas_be
    meldung=['The number of alphas doesn''t fit. There should be ' num2str(NoAlphas_theo) ', but there are ' num2str(NoAlphas_be) '. Do you wish to run the simulation using the default value for all alphas?.'];
    answer= questdlg(meldung, 'Miscount!', 'Yes', 'Cancel', 'Yes');
    if strcmp('Yes', answer);
        paramsREM.alpha=repmat(paramsREM.alphadefault, 1, NoAlphas_theo);
    else
        return
    end
end

%expand alphas matching noWeights for each single stimulus
paramsREM.alphas=[];
k=0;
for i =reshape([1:paramsREM.noWeights],noWeights,noCS)
   k=k+1; 
   paramsREM.alphas(i)= paramsREM.alpha(k);
end
    




%(1) Für replaced elements nach Medin-Estes brauchen wir
% die Proportionen der Ersetzungen, die wir aus einer Datei
% einlesen
proFileID = fopen(paramsREM.proportionPath, 'rt');
if(proFileID == -1)
    msgbox('The matrix with the proportions can''t be read! ', 'Error', 'error', 'modal');
    return;
end
tmp = zeros(1, noCS - 1);
proportions = zeros(noCS, 2^(noCS - 1));

for i = 1:noCS
    nextLine = fgetl(proFileID);
    %check if proportion matrix is too small
    if ~ischar(nextLine)
        msgbox('The matrix with the proportions has not enough lines! ', 'Error', 'error', 'modal');
        return;
    end
    paramsREM.r=str2num(nextLine);
    paramsREM.r=paramsREM.r(1,1);
    for j = 1:(noCS - 1)
        [token, nextLine] = strtok(nextLine);
        %check if proportion matrix is too small
        if isempty(token)
            msgbox('The matrix with the proportions has not enough columns! ', 'Error', 'error', 'modal');
            paramsREM=[];
            return;
        end
        tmp(j) = str2num(token);
    end
    %check if proportion matrix is too large
    if ~isempty(nextLine)
        msgbox('The matrix with the proportions has too many values! ', 'Error', 'error', 'modal');
        paramsREM=[];
        return;
    end
    proportions(i, :) = getProps(tmp);

end


%(3) Anzahl der Lernmuster bestimmen und eine entsprechende
%Matrix anlegen.
noPat = size(paramsGen.code,1);
paramsREM.pattern = zeros(noPat, paramsREM.noWeights);
tmp = paramsGen.code;

%(4)nacheinander alle Muster einlesen und so expandieren,
%daß sie dem Netz als Eingabe dienen können.
for i = 1:noPat
    %compute 0/1 matrix
    paramsREM.pattern(i, 1:paramsREM.noWeights) = expandInpEstes(tmp(i,:), paramsREM.noWeights, proportions);
    % weight with alphas
    paramsREM.pattern(i, 1:paramsREM.noWeights) = paramsREM.pattern(i, 1:paramsREM.noWeights).*paramsREM.alphas;
end




%Anlegen des Testpattern
paramsREM.testpattern=paramsREM.pattern;




function binom = binomCoeff(n, m)
binom = prod([(m+1):n])/prod(1:(n-m)); %#ok<NBRAK>



function props = getProps(arg)
lArg = length(arg); % no of components -1 
arg = arg(lArg:-1:1); % replaced stuff
count = 2^lArg;
props = zeros(1, count);
for i = 1:count
    bin = dec2bin(i-1,lArg);
    accu = 1;
    for j = lArg:-1:1
        if strcmp(bin(j), '0')
            accu = accu * (1 - arg(j));
        else
            accu = accu * arg(j);
        end
    end
    props(i) = accu;
end

function expInp = expandInpEstes(csArray, noW, proportions)
%Diese Methode erzeugt aus einem gegebenen Muster-Vektor
%in dem die An- und Abwesenheit der CS kodiert ist, einen
%der Netzspezifikation entsprechenden Input Vektor. Dabei
%bilden die Überlegungen von Wagner (2003, Medin-Estes) die
%Grundlage für die Expansion

%Ein Feld der entsprechenden Länge anlegen, auf dem man
%dann arbeiten kann.
expInp = [];

noCS = length(csArray);

%Die Anzahl der Elemente pro CS
UpCS = noW/noCS;

for i = 1:noCS;
    aktCS = csArray(i);
    restCS = [csArray(1:(i-1)), csArray((i+1):noCS)];
    lRest = noCS-1;
    restCS = restCS(lRest:-1:1);
    offset = (i-1) * UpCS;

    %Wenn der CS nicht gesetzt ist sind alle seine Elemente Null
    if ~aktCS
        expInp = [expInp, zeros(1, UpCS)];
        %andernfalls muessen die Units mit den passenden Werten versehen werden
    else
        %der erste ist einfach
        CSinp = [proportions(i, 1)];
        iterations = 2^lRest;
        %Für den Rest ist ein wenig Index-Arithmetik nötig...
        %Durch die Umwandlung in Binärzahlen ist eine Zuordnung von
        %anwesenden/abwesenden CS zu Positionen im Array der Input-
        %Elemente für den jeweiligen CS möglich.
        for j = 2:iterations
            %Welche von den restlichen CS werden an
            %der Position j im proportions-Array berücksichtigt?
            bin = dec2bin(j-1, lRest);
            binArray = convertToArray(bin, lRest);
            %Diese CS werden aus dem csArray abgerufen. Abhängig
            %davon, welcher dieser CS gesetzt ist wird (wiederum
            %mit Hilfe von Binärzahlen) das passende Element gesetzt.
            subCS = restCS(binArray > 0);
            lSubCS = length(subCS);
            tmp = zeros(1, 2^lSubCS);
            subCSbin = convertToBin(subCS, lSubCS);
            tmp(bin2dec(subCSbin) + 1) = proportions(i, j);
            %Jetzt noch die Elemente des aktuellen CS entsprechend erweitern
            CSinp = [CSinp, tmp];
        end
        expInp = [expInp, CSinp];
    end
end

function bA = convertToArray(bin, lBin)
bA = zeros(1, lBin);
for i = 1:lBin
    bA(i) = str2num(bin(i));
end

function b = convertToBin(vec, lVec)
b = [];
for i = 1:lVec
    b = [b, num2str(vec(i))];
end



%-------------------------------------------------------------------------------
%     This file is part of ALTSim.
% 
%     ALTSim is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ALTSim is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with ALTSim.  If not, see <http://www.gnu.org/licenses/>.