function paramsINM=trainINM(paramsGen, paramsINM)
%simulation, ploting and saving of the Inhibted Elements Model


%The generel parameters passed by paramsGen are
% Stimulusname: name of all trial types: cell
% Testname:     "
% Kont:         US lambda for all trial types: noTrialTypes-vector
% KontTest:     "
% beta:         US beta for all trial types: noTrialTypes-vector
% AnzVp:        number of sequences: double
% CS:           CS-sequence: noTrial*AnzVp-matrix
% phaseInfo     phase of current trial:  noTrial-vector 
% US:           US-sequence: noTrial*AnzVp-matrix
% block:        trials trained in each block in each phase: cell matrix for each phase
% code:         CS for each trial types (without unique cue or similar): noTrialTypes*noCS-matrix
% testcode:     "


%paramsINM         structure with PARAMETER for Inhibited Elements Model
% resultsPath:      directory where resulting matrices are saved
% startingWeights:  txt-file with associative weights whose last line is used as starting
% alpha:            vector with alpha parameter
% nn:               amount of normalisation
% pattern:          expanded stimulus code
% noWeights:        number of elements needed to represent all stimuli combinations
% rule:             learning rule of Schultheis et al or Glautier?
% weightPlot:       checkbox for weightPlot
% activationPlots:  checkbox for activatiaonPlots

%Important Output Variables: 
% amatrix           US activations by CSs of current trial, last entry on third dimension is average across sequences: noTrials*noTrialTypes*(AnzVp+1)-matrix
% TestAktivierung   Hypothetical US activations by CSs of different trial types, last entry on third dimension is average across sequences: noTrials*noTrialTypes*(AnzVp+1)-matrix
% MeanWeights       associative weights for each element, average across sequences: (noTrials+1)*(no elements)-matrix

% Copyright (C) 2010-2017  Anna Thorwart

% check parameter and prepare matrices
if isempty(paramsINM)
    errordlg('Something is missing.')
    return
end

noPattern=size(paramsGen.code,1);
noStimuli=size(paramsGen.code,2);
noTrials = size(paramsGen.CS,1);


% check alphas
NoAlphas_theo=noStimuli; %number of alphas nedded
NoAlphas_be=size(paramsINM.alpha,2); %number of alphas entered

if NoAlphas_theo ~=NoAlphas_be
    meldung=['The number of alphas doesn''t fit. There should be ' num2str(NoAlphas_theo) ', but there are ' num2str(NoAlphas_be) '. Do you wish to run the simulation using the default value for all alphas?.'];
    answer= questdlg(meldung, 'Miscount!', 'Yes', 'Cancel', 'Yes');
    if strcmp('Yes', answer);
        paramsINM.alpha=repmat(paramsINM.alphadefault, 1, NoAlphas_theo);
    else
        return
    end
end

% stimulus representation

% compute number of units necesarry to represent each stimulus
elemts=ff2n(noStimuli); % all combinations of stimuli
elemts=elemts(elemts(:,1)==1,:); % delete those where first stimulus, i.e. the stimulus the units belong, is NOT activated
StringLength=sum(elemts,2); % number of stimuli coded in the unit
paramsINM.noWeights=size(elemts,1);  % no of units necesearry to represent all combinations, or so that each combination can inhibit one singel unit

% compute activation (proportion) of units inhibited by stimuli combinations consisting of different number of stimui (=StringLength)
% in representation of single stimulus so that the
% overall activation of all stimuli corresponds to the amount of normalisation specified with paramsINM.nn When nn=1, the activation is always 1(complete
% normalisation). When nn=0, the first unit, that is never inhibited, is
% completely activated to 1.
proportions=zeros(1,paramsINM.noWeights);
for jj=1:noStimuli
    % activation of all elements inhibted by a stimulus combination with jj CS present 
    proportions(StringLength==jj)= 1/(noStimuli-(jj-1))^paramsINM.nn - sum(proportions(~any(elemts(:,2:(noStimuli+1-jj))==1,2)));
end


% expand code based on computed proportions and stimulus pattern presented
% in experiment
paramsINM.pattern=zeros(noPattern, paramsINM.noWeights*noStimuli);

for jj= 1:noPattern
    for kk=1:noStimuli
        if paramsGen.code(jj,kk)
            patt = [paramsGen.code(jj,kk) paramsGen.code(jj,1:(kk-1)) paramsGen.code(jj,(kk+1):noStimuli)];
            % activation of this CS in this pattern & activation of remaining, ie. context, stimuli in this patternactTemp=[];
            indCS=find(patt>0);
            paramsINM.pattern(jj,(kk-1)*paramsINM.noWeights+1 : kk*paramsINM.noWeights)=proportions.*(~any(elemts(:,indCS(2:end))==1,2))';
        end
    end
    
end

paramsINM.testpattern=paramsINM.pattern;
noT = size(paramsINM.testpattern, 1);
disp('simulating model...')

noInp = paramsINM.noWeights*noStimuli;
alpha=reshape(repmat(paramsINM.alpha,paramsINM.noWeights,1),1,noInp); %expand alpha

meanWeights=zeros(noTrials+1, noInp);

% output activation by each pattern in each trial
TestActivation(noTrials,noPattern,paramsGen.AnzVp)=0;
% output activation by pattern presented in that trial
Output=zeros(noTrials, paramsGen.AnzVp);


for s=1:paramsGen.AnzVp
    
    
    %%Anfangswerte setzen
    path=paramsINM.startingWeights;
    % associative weights
    if isempty(path)|| strncmpi(paramsINM.startingWeights,'"',1)
        Ass_Str = zeros(1, noInp);
    else % TODO: im Moment fuer jede Vp die gleichen Startgewichte
        Ass_Str=textread(path, '%s', 'bufsize', 100000095, 'delimiter','\n', 'whitespace',' ');
        Ass_Str=str2num(char(Ass_Str(size(Ass_Str,1),:)));
    end
    
    
    for jj=1:noTrials
        TestActivation(jj,:,s)=(paramsINM.pattern*Ass_Str(jj,:)')';
        Output(jj,s)=paramsINM.pattern(paramsGen.CS(jj,s),:)*Ass_Str(jj,:)';
        error=paramsGen.US(jj,s)-Output(jj,s);
        beta = paramsGen.beta(paramsGen.CS(jj,s));
        Ass_Str(jj+1,:)=Ass_Str(jj,:)+(logical(paramsINM.pattern(paramsGen.CS(jj,s),:))'.*alpha'.*beta*(error))';
        
    end
    
    meanWeights=meanWeights+Ass_Str; % add assocative weights to average later across sequences
    OutputVp=Output(:,s)';
    [amatrix(:,:,s)]=sorttrials(OutputVp, paramsGen.CS(:,s), paramsGen.phaseInfo, length(paramsGen.Stimulusname));
end

amatrix(:,:,s+1)=mean(amatrix,3);
TestActivation(:,:,s+1)=mean(TestActivation,3);
meanWeights= meanWeights/paramsGen.AnzVp;

%% plotting

%create string with training stimulus
trainingstring=[];
for ii=1:length(paramsGen.block)
    trainingstring = [trainingstring, strjoin(paramsGen.Stimulusname(paramsGen.block{ii})',  ','),'|'];
end

% create string with parameters

if length(paramsGen.beta)<=7
    titlestring ={['normalisation: ', num2str(paramsINM.nn),';'];...
        ['\alpha=', mat2str(paramsINM.alpha'),'; \beta=', mat2str(paramsGen.beta),'; phases: ', trainingstring]};
else
    titlestring ={['learning rule: ', num2str(paramsINM.nn),';'];...
        ['\alpha=', mat2str(paramsINM.alpha'),'; \beta=xxx ; phases: ', trainingstring]};
end

if paramsINM.weightPlot
    plotWeigthCourse(meanWeights(:,:),'IEM: Associative weights of elements ', titlestring);
end

if paramsINM.activationPlots
    plotOutputActivation(TestActivation(:,:,s+1),paramsGen.phaseInfo,'IEM: Hypothetical US activations by CSs of different trial types', titlestring,paramsGen.Stimulusname );
   
end
if paramsINM.trainedPlot
    plotAmatrix(amatrix(:,:,s+1),paramsGen.phaseInfo,'IEM: US activations by CSs of current trial', titlestring, [paramsGen.Stimulusname]);
end

%% saving

speicherpath=paramsINM.resultsPath;

if ~isempty(speicherpath{2})
    if isempty(speicherpath{1}) %if not flder is specified, use current folder
        speicherpath{1}=pwd;
    end
    dlmwrite (fullfile(speicherpath{1},[speicherpath{2}, '_testactivation.txt']), [TestActivation(:,:,s+1), paramsGen.phaseInfo], '\t') ;
    dlmwrite (fullfile(speicherpath{1},[speicherpath{2}, '_trainedactivation.txt']), [amatrix(:,:,s+1), paramsGen.phaseInfo], '\t')
    dlmwrite (fullfile(speicherpath{1},[speicherpath{2}, '_weights.txt']),meanWeights(:,:),'\t') ;

end
%-------------------------------------------------------------------------------
%     This file is part of ALTSim.
% 
%     ALTSim is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ALTSim is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with ALTSim.  If not, see <http://www.gnu.org/licenses/>.
