function paramsP=trainP(paramsGen, paramsP)
% simulation, ploting and saving of the Pearce Model and its extension


%The generel parameters passed by paramsGen are
% Stimulusname: name of all trial types: cell
% Testname:     "
% Kont:         US lambda for all trial types: noTrialTypes-vector
% KontTest:     "
% beta:         US beta for all trial types: noTrialTypes-vector
% AnzVp:        number of sequences: double
% CS:           CS-sequence: noTrial*AnzVp-matrix
% phaseInfo     phase of current trial:  noTrial-vector
% US:           US-sequence: noTrial*AnzVp-matrix
% block:        trials trained in each block in each phase: cell matrix for each phase
% code:         CS for each trial types (without unique cue or similar): noTrialTypes*noCS-matrix
% testcode:     "

%paramsP:          parameter for Pearce simulation
% d                 generalization parameter (Kinder & Lachnit, 2001)
% resultsPath:      directory where resulting matrices are saved
% startingWeights:  txt-file with associative weights whose last line is used as starting
% alpha             vector with alpha parameter for each configural unit
% alphadefault      default value for alpha definded in default.conf
% alphasinput       vecotor with alpha for each inpt unit
% weightPlot:       checkbox for weightPlot
% activationPlots:  checkbox for activatiaonPlots

%Important Output Variables: last entry on third dimension is always mean across sequences
% amatrix           US activations by CSs of current trial: noTrials*noTrialTypes*(AnzVp+1)-matrix
% TestAktivierung   Hypothetical US activations by CSs of different trial types: noTrials*noTrialTypes*(AnzVp+1)-matrix
% weights           associative weights for each trial type: (noTrials+1)*noTrialTypes*(AnzVp+1)-matrix
% weightsSum        associative weights for each distinct configuration:% (noTrials+1)*no configurations (see line 132) *(AnzVp+1)-matrix 


% Copyright (C) 2006-2017  Anna Thorwart and Holger Schultheis

%check alpha for input units;
NoAlphas_theo=size(paramsGen.code,2); %number of alphas necessary
NoAlphas_be=size(paramsP.alphasinput,2); %number of alphas specified
if NoAlphas_theo ~=NoAlphas_be
    meldung=['The number of alphas for input units doesn''t fit. There should be ' num2str(NoAlphas_theo) ', but there are ' num2str(NoAlphas_be) '. Do you wish to run the simulation using the default value for all alphas?'];
    answer= questdlg(meldung, 'Miscount!', 'Yes', 'Cancel', 'Yes');
    if strcmp('Yes', answer);
        paramsP.alphasinput=repmat(paramsP.alphadefault, 1, NoAlphas_theo);
    else
        return
    end
end

%weight stimulus code wiht alphas
code=paramsGen.code.*repmat(paramsP.alphasinput,[size(paramsGen.code,1) 1]);  %activation of input units

%test stimuli
testcode=code;
noT=size(testcode,1);


%check alpha for configural units;
NoAlphas_theo=size(paramsGen.code,1); %number of alphas necessary
NoAlphas_be=size(paramsP.alpha,2); %number of alphas specified
if NoAlphas_theo ~=NoAlphas_be
    meldung=['The number of alphas for the configural units doesn''t fit. There should be ' num2str(NoAlphas_theo) ', but there are ' num2str(NoAlphas_be) '. Do you wish to run the simulation using the default value for all alphas?'];
    answer= questdlg(meldung, 'Miscount!', 'Yes', 'Cancel', 'Yes');
    if strcmp('Yes', answer);
        paramsP.alpha=repmat(paramsP.alphadefault, 1, NoAlphas_theo);
        
    else
        return
    end
end
alpha_reiz = paramsP.alpha;
SIMPAR=paramsP.d;

%Vergleichs-Matrix fuer die Aktivierung der Konfigural-Units=Aktivierte Input-Units=Elemente
Vgl=code; %activation of input units
KonfigUnits=size(Vgl,1); %number of configural units


% initialise matrix for associative weights  of all subjects
weights=zeros(size(paramsGen.CS,1),size(Vgl,1), paramsGen.AnzVp);


%% for each subject
for ss=1:paramsGen.AnzVp
    
    CScodeVp=code(paramsGen.CS(:,ss),:);
    USVP=paramsGen.US(:,ss);
    
    % starting weights
    if isempty(paramsP.startingWeights) || strncmpi(paramsP.startingWeights,'"',1)
        Ass_Str =zeros(size(Vgl,1),1);
    else
        path = paramsP.startingWeights;
        startingWeights=dlmread(path); %read in weights matrix
        Ass_Str= startingWeights(end,:)' ;% set last line as starting weights
        weights(1,:,ss)=startingWeights(end,:); % write starting weights in weights matrices
    end
    
    
    Trials=size(CScodeVp,1); %number of trials
    
    
    % learning
    for jj=1:Trials
        normCScodeVp=CScodeVp(jj,:)'./norm(CScodeVp(jj,:)');  %activation of output units by presented CS in trial j
        for k=1:KonfigUnits
            normVgl=Vgl(k,:)'./norm(Vgl(k,:)'); %activation of output units by CS k (=connection strength between output and config unit k)
            simP(k)=(normCScodeVp'*normVgl)^SIMPAR;  %activation of configural unit k  by presented CS in trial j
        end
        Output(jj,ss)=simP*Ass_Str; %activation of the US unit by presented CS in trial j of sequence/subject s
        Fehler=USVP(jj)-Output(jj,ss); %error
        beta = paramsGen.beta(paramsGen.CS(jj,ss));
        alpha = alpha_reiz(paramsGen.CS(jj,ss));
        
        for t=1:noT;
            simPTest=[];
            normtestcode=testcode(t,:)'./norm(testcode(t,:)');%activation of output units by test trial k (=connection strength between output and config units)
            for k=1:KonfigUnits;
                normVgl=Vgl(k,:)'./norm(Vgl(k,:)');
                simPTest(k)=(normtestcode'*normVgl)^SIMPAR;  %activation of configural unit k  by test trial t
            end
            TestAktivierung(jj,t,ss)=simPTest*Ass_Str; %activation of the US by CS in test trial t at the beginning of trial j
        end
        
        
        
        Selektion=zeros(size(simP));
        Selektion(paramsGen.CS(jj,ss))=1; %which configural-unit is the winner that learns
        %NOTE: if  exactly the same stimulus is used in several different trial types(A+, A-, A?), there is
        %one unit for each trial type within the simulation. It even doesn't matter if these trials
        %are training or only test trials. 
        %As all are completely activated in appropiate trials, there is perfect "generalisation"
        %of their already learned associations (Output computations at line 112 and Testaktivierung on line 124).
        %However, only the associative weight of the presented trial type unit is changed.
        %This was different in ALTSim versions lower than 3.5. There, associative weights of all
        %units whose activation was 1 changed. Therefore, learning was unappropriatly accelerated for
        %those stimuli configurations presented in several different trial types.
        
        Ass_Str=Ass_Str+(Selektion'*alpha*beta*(Fehler));
        weights(jj+1,:,ss)=Ass_Str';
    end
    OutputVp=Output(:,ss)';
    
    [amatrix(:,:,ss)]=sorttrials(OutputVp, paramsGen.CS(:,ss), paramsGen.phaseInfo,length(paramsGen.Stimulusname));
    
    
    % sum up the final associatve weights of configural units belonging to the same CS configuration across
    % training
    [b, ~, n] = unique(Vgl,'rows'); %get all unique configuration
    for ii=1:size(b,1) %for each disctint configuration
        weightsSum(:,ii,ss)=sum(weights(:,n==ii,ss),2); %sum up the associative strength for identical configuration for plotting
        legNameSum{ii,:}=[paramsGen.Stimulusname{n==ii}]; % create names for  distinct configuarations based on name of trial types
    end
end

%% average over subjects
amatrix(:,:,ss+1)=mean(amatrix,3);
TestAktivierung(:,:,ss+1)=mean(TestAktivierung,3);
weights(:,:,ss+1)= mean(weights,3);
weightsSum(:,:,ss+1)= mean(weightsSum,3);



%% plotting

%create string with training stimulus
trainingstring=[];
for ii=1:length(paramsGen.block)
    trainingstring = [trainingstring, strjoin(paramsGen.Stimulusname(paramsGen.block{ii})', ','),'|'];
end

% create string with parameters

if size(paramsP.alpha,2)<7 & length(paramsGen.beta) < 7
    titlestring ={['d = ', num2str(paramsP.d), '; {\alpha}_{input}=', mat2str(paramsP.alphasinput'),';\alpha_{configural}=', mat2str(paramsP.alpha')];...
        ['\beta=', mat2str(paramsGen.beta), '; phases: ', trainingstring]};
elseif size(paramsP.alpha,2) > 7 & length(paramsGen.beta) <= 7
    titlestring ={['d = ', num2str(paramsP.d),  '; {\alpha}_{input}=', mat2str(paramsP.alphasinput'),';'];...
        ['\alpha_{configural}=xxx ; \beta=', mat2str(paramsGen.beta), '; phases: ', trainingstring]};
elseif size(paramsP.alpha,2)<=7 && length(paramsGen.beta) > 7
    titlestring ={['d = ', num2str(paramsP.d),  '; {\alpha}_{input}=', mat2str(paramsP.alphasinput'),';'];...
        ['\alpha_{configural}=', mat2str(paramsP.alpha'),'; \beta=xxx ; phases: ', trainingstring]};
else
    titlestring ={['d = ', num2str(paramsP.d), ';'];...
        ['\alpha_{configural}=xxx; \beta=xxx ; phases: ', trainingstring]};
end



if paramsP.weightPlot
    plotWeigthCourse(weightsSum(:,:,ss+1),'Pearce: Associative weights of configural units', titlestring);
    legend(legNameSum)
end
if paramsP.activationPlots
    plotOutputActivation(TestAktivierung(:,:,ss+1),paramsGen.phaseInfo,'Pearce: Hypothetical US activations by CSs of different trial types', titlestring,paramsGen.Stimulusname );
end
if paramsP.trainedPlot
    plotAmatrix(amatrix(:,:,ss+1),paramsGen.phaseInfo,'Pearce: US activations by CSs of current trial', titlestring, [paramsGen.Stimulusname]);
end

%% Saving
speicherpath=paramsP.resultsPath;
if ~isempty(speicherpath{2});
    if isempty(speicherpath{1}) %if not flder is specified, use current folder
        speicherpath{1}=pwd;
    end
    dlmwrite (fullfile(speicherpath{1},[speicherpath{2}, '_testactivation.txt']), [TestAktivierung(:,:,ss+1), paramsGen.phaseInfo], '\t') ;
    dlmwrite (fullfile(speicherpath{1},[speicherpath{2},  '_trainedactivation.txt']), [amatrix(:,:,ss+1), paramsGen.phaseInfo], '\t')
    dlmwrite (fullfile(speicherpath{1},[speicherpath{2},  '_weights.txt']),weights(:,:, ss+1),'\t') ;
end
%-------------------------------------------------------------------------------
%     This file is part of ALTSim.
% 
%     ALTSim is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ALTSim is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with ALTSim.  If not, see <http://www.gnu.org/licenses/>.