function paramsREM=trainREM(paramsGen, paramsREM)
%simulation, ploting and saving of the Replaced Elements Model


%The generel parameters passed by paramsGen are
% Stimulusname: name of all trial types: cell
% Testname:     "
% Kont:         US lambda for all trial types: noTrialTypes-vector
% KontTest:     "
% beta:         US beta for all trial types: noTrialTypes-vector
% AnzVp:        number of sequences: double
% CS:           CS-sequence: noTrial*AnzVp-matrix
% phaseInfo     phase of current trial:  noTrial-vector 
% US:           US-sequence: noTrial*AnzVp-matrix
% block:        trials trained in each block in each phase: cell matrix for each phase
% code:         CS for each trial types (without unique cue or similar): noTrialTypes*noCS-matrix
% testcode:     "


%paramsREM:         parameter for Replaced Elements Model
% resultsPath:      directory where resulting matrices are saved
% startingWeights:  txt-file with associative weights whose last line is used as starting
% alpha             vector with alpha parameter
% proportionPath    txt-file with replaced proportions
% pattern:          expanded stimulus code
% noWeights:        number of elements needed to represent all stimuli combinations
% rule:             learning rule of Schultheis et al or Glautier?
% weightPlot:       checkbox for weightPlot
% activationPlots:  checkbox for activatiaonPlots

% Important Output Variables:
% amatrix           US activations by CSs of current trial,  last entry on third dimension is average across sequences: noTrials*noTrialTypes*(AnzVp+1)-matrix
% TestAktivierung   Hypothetical US activations by CSs of different trial types, last entry on third dimension is average across sequences: noTrials*noTrialTypes*(AnzVp+1)-matrix
% MeanWeights       associative weights for each element, average across sequences: (noTrials+1)*(no elements)-matrix



% Copyright (C) 2006-2017  Anna Thorwart and Holger Schultheis



% check parameter and prepare matrices
paramsREM= initEstes(paramsGen, paramsREM);
if isempty(paramsREM)
    errordlg('Something is missing.')
    return
end





disp('simulating model...')


%Anzahl der Lerndurchgï¿½nge, Muster, Input-Units und Lernparameter holen
noTrials = size(paramsGen.CS,1); %trials
noInp = paramsREM.noWeights;
noT = size(paramsREM.testpattern, 1);
weights = zeros(noTrials, noInp);
meanWeights=zeros(noTrials+1, noInp);





for s=1:paramsGen.AnzVp

    rpatternVp = paramsREM.pattern(paramsGen.CS(:,s),:);
    USVP=paramsGen.US(:,s);
    
    %%Anfangswerte setzen
    path=paramsREM.startingWeights;

    if isempty(path)|| strncmpi(paramsREM.startingWeights,'"',1)
        Ass_Str = zeros(1, paramsREM.noWeights);
    else % TODO: im Moment fuer jede Vp die gleichen Startgewichte
        Ass_Str=textread(path, '%s', 'bufsize', 100000095, 'delimiter','\n', 'whitespace',' ');
        Ass_Str=str2num(char(Ass_Str(size(Ass_Str,1),:)));
    end

    weights(1,:)=Ass_Str;

    for j = 1:noTrials
        inp  = rpatternVp(j,:);

        if paramsREM.rule == 1 %marburger way

            for k = 1:noT
                TestAktivierung(j,k,s) = Ass_Str * paramsREM.testpattern(k, 1:noInp)';
            end
            Output(j,s)   = Ass_Str*inp';
            Fehler=USVP(j)-Output(j,s);
            beta = paramsGen.beta(paramsGen.CS(j,s));

            Ass_Str = Ass_Str + (beta* (Fehler) * (inp > 0));


        elseif paramsREM.rule == 2 %southamptoner way

            for k = 1:noT
                TestAktivierung(j,k,s) = sum(Ass_Str(paramsREM.testpattern(k, 1:noInp)>0));
            end

            Output(j,s)   = sum(Ass_Str(inp>0)); %#ok<*AGROW>
            Fehler=USVP(j)-Output(j,s);
            beta = paramsGen.beta(paramsGen.CS(j,s));

            Ass_Str = Ass_Str + inp.*(beta* (Fehler) * (inp > 0));

        end


        weights(j+1,:)=Ass_Str;

    end
   
    meanWeights=meanWeights+weights;
    OutputVp=Output(:,s)';
    [amatrix(:,:,s)]=sorttrials(OutputVp, paramsGen.CS(:,s), paramsGen.phaseInfo, length(paramsGen.Stimulusname));
end

amatrix(:,:,s+1)=mean(amatrix,3); % averaged across subjects
TestAktivierung(:,:,s+1)=mean(TestAktivierung,3);
meanWeights= meanWeights/paramsGen.AnzVp;

%% ploting


%create string with training stimulus
trainingstring=[];
for ii=1:length(paramsGen.block)
    trainingstring = [trainingstring, strjoin(paramsGen.Stimulusname(paramsGen.block{ii})',  ','),'|'];
end
% create string with parameters
rulestring={'Schultheis et al. 2008'; 'Glautier 2007'};
if length(paramsGen.beta)<=7
titlestring ={['learning rule: ', rulestring{paramsREM.rule},', r = ' , num2str(paramsREM.r),';'];...
    ['\alpha=', mat2str(paramsREM.alpha'),'; \beta=', mat2str(paramsGen.beta),'; phases: ', trainingstring]};
else
    titlestring ={['learning rule: ', rulestring{paramsREM.rule},', r = ' , num2str(paramsREM.r),';'];...
    ['\alpha=', mat2str(paramsREM.alpha'),'; \beta=xxx ; phases: ', trainingstring]};
end

if paramsREM.weightPlot
    plotWeigthCourse(meanWeights(:,:),'REM: Associative weights of elements ', titlestring);
end

if paramsREM.activationPlots
    plotOutputActivation(TestAktivierung(:,:,s+1),paramsGen.phaseInfo,'REM: Hypothetical US activations by CSs of different trial types', titlestring,paramsGen.Stimulusname );
end
if paramsREM.trainedPlot
    plotAmatrix(amatrix(:,:,s+1),paramsGen.phaseInfo,'REM: US activations by CSs of current trial', titlestring, [paramsGen.Stimulusname]);
end
%% saving

speicherpath=paramsREM.resultsPath;

if ~isempty(speicherpath{2});
    if isempty(speicherpath{1}) %if not flder is specified, use current folder
        speicherpath{1}=pwd;
    end
    dlmwrite (fullfile(speicherpath{1},[speicherpath{2}, '_testactivation.txt']),[TestAktivierung(:,:,s+1), paramsGen.phaseInfo], '\t') ;
    dlmwrite (fullfile(speicherpath{1},[speicherpath{2}, '_trainedactivation.txt']),  [amatrix(:,:,s+1), paramsGen.phaseInfo], '\t')
    dlmwrite (fullfile(speicherpath{1},[speicherpath{2}, '_weights.txt']),meanWeights(:,:),'\t') ;
end
%-------------------------------------------------------------------------------
%     This file is part of ALTSim.
% 
%     ALTSim is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ALTSim is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with ALTSim.  If not, see <http://www.gnu.org/licenses/>.
