function paramsRW=trainRW(paramsGen, paramsRW)
%simulation, ploting and saving of the Rescorla Wagner Model and its extensions


% The generel parameters passed by paramsGen are
% Stimulusname: name of all trial types: cell
% Testname:     "
% Kont:         US lambda for all trial types: noTrialTypes-vector
% KontTest:     "
% beta:         US beta for all trial types: noTrialTypes-vector
% AnzVp:        number of sequences: double
% CS:           CS-sequence: noTrial*AnzVp-matrix
% phaseInfo     phase of current trial:  noTrial-vector 
% US:           US-sequence: noTrial*AnzVp-matrix
% block:        trials trained in each block in each phase: cell matrix for each phase
% code:         CS for each trial types (without unique cue or similar): noTrialTypes*noCS-matrix
% testcode:     "




%paramsRW:         parameter for Rescorla-Wagner simulation
% UC                compute unique cues
% mUC               use salience equitation of Pearce (1994)
% resultsPath:      directory where resulting matrices are saved
% startingWeights:  txt-file with associative weights whose last line is used as starting
% alpha             vector with alpha parameter
% alphadefault:     default value defined in default.conf
% weightPlot:       checkbox for weightPlot
% activationPlots:  checkbox for activatiaonPlots


%Important Output Variables: last entry on third dimension is always mean across sequences
% amatrix           US activations by CSs of current trial: noTrials*noTrialTypes*(AnzVp+1)-matrix
% TestAktivierung   Hypothetical US activations by CSs of different trial types: noTrials*noTrialTypes*(AnzVp+1)-matrix
% weights           associative weights for each CS + unique: (noTrials+1)*(no CS+unique cues)*(AnzVp+1)-matrix


% Copyright (C) 2006-2017  Anna Thorwart and Holger Schultheis




% prepare matrices
% all stimuli
if paramsRW.UC == 1 %unique cue hypotheses
    code=expandMatrix_UC(paramsGen.code);
else  % orignal Rescorla-Wagner Model
    code= paramsGen.code;
end

% test stimuli
testcode=code;
noT=size(testcode,1); 

% check alpha

NoAlphas_theo=size(code,2); %Anzahl der Alphas, die benï¿½tigt werden
NoAlphas_be=size(paramsRW.alpha,2); %Anzahl der Alphas, die eingegeben sind

if NoAlphas_theo ~=NoAlphas_be
    meldung=['The number of alphas doesn''t fit. There should be ' num2str(NoAlphas_theo) ', but there are ' num2str(NoAlphas_be) '. Do you wish to run the simulation using the default value for all alphas?.'];
    answer= questdlg(meldung, 'Miscount!', 'Yes', 'Cancel', 'Yes');
    if strcmp('Yes', answer);
        paramsRW.alpha=repmat(paramsRW.alphadefault, 1, NoAlphas_theo);
    else
        return
    end
end
alpha = paramsRW.alpha;

%initialize weights matrix
weights=zeros(size(paramsGen.CS,1),size(code,2),paramsGen.AnzVp);


for ss=1:paramsGen.AnzVp

    CScodeVp=code(paramsGen.CS(:,ss),:);
    USVP=paramsGen.US(:,ss);

    % initialize weights

    if isempty(paramsRW.startingWeights) || strncmpi(paramsRW.startingWeights,'"',1)% no starting weights are defined
        Ass_Str =zeros (size(CScodeVp,2),1); % start with zeros
    elseif ~isempty(paramsRW.startingWeights); %TODO; catch missing file
        path=paramsRW.startingWeights;
        startingWeights=dlmread(path); %read in weights matrix
        Ass_Str= startingWeights(end,:)' ;% set last line as starting weights
        weights(1,:,ss)=startingWeights(end,:); % write starting weights in weights matrices
    end

    % Festsetzen, ob Unique Cues in den Salienzbruch miteinbezogen werden
    Salienzgewicht=zeros(1,size(CScodeVp,2)); % initialisieren

%     if paramsRW.mUC == 3  % choose if unique cues are included in salience's fraction 
%         Salienzgewicht=ones(1,size(CScodeVp,2)); % alle auf eins setzen
%     elseif paramsRW.mUC ==2
        Salienzgewicht(1:size(paramsGen.code,2))= 1; % nur die Elemente auf eins setzen
%     end

    % training
    Trials=size(CScodeVp,1);
    for jj=1:Trials

        for k=1:noT                                                    %Aktivierung bei den verschiednen Testreizen
            TestAktivierung(jj,k,ss)=testcode(k,:)*Ass_Str   ;            %j trials. k, teststimuli, s subject
        end

        Output(jj,ss)=CScodeVp(jj,:)*Ass_Str;                                    %Aktivierung bei dem gezeigten CS
        Fehler=USVP(jj)-Output(jj,ss);
        beta = paramsGen.beta(paramsGen.CS(jj,ss));

        if paramsRW.mUC 
            alphagew=alpha/sum(Salienzgewicht.*CScodeVp(jj,:).*alpha) ;          %%%%ALPHA
            Ass_Str=Ass_Str+(CScodeVp(jj,:)'.*alpha'.*alphagew'*beta*(Fehler));
        else
            Ass_Str=Ass_Str+(CScodeVp(jj,:)'.*alpha'*beta*(Fehler));             %Lernen
        end
        weights(jj+1,:,ss)=Ass_Str';
    end

     [amatrix(:,:,ss)]=sorttrials(Output(:,ss)', paramsGen.CS(:,ss), paramsGen.phaseInfo, length(paramsGen.Stimulusname));

end

%average over subjects
weights(:,:,ss+1)= mean(weights,3);
TestAktivierung(:,:,ss+1)=mean(TestAktivierung,3);
amatrix(:,:,ss+1)=mean(amatrix,3);



%% ploting

%create string with training stimulus
trainingstring=[];
for ii=1:length(paramsGen.block)
    trainingstring = [trainingstring, strjoin(paramsGen.Stimulusname(paramsGen.block{ii})', ','),'|'];
end

% create string with parameters
if size(paramsRW.alpha,2)<7 & length(paramsGen.beta) < 7
    titlestring ={['unique cues:' num2str(paramsRW.UC), ',  modified salience:', num2str(paramsRW.mUC),';'];...
        ['\alpha=', mat2str(paramsRW.alpha'),'; \beta=', mat2str(paramsGen.beta), '; phases: ', trainingstring]};
elseif size(paramsRW.alpha,2) >7 & length(paramsGen.beta) <= 7
    titlestring= {['unique cues:' num2str(paramsRW.UC), ',  modified salience:', num2str(paramsRW.mUC),';'];...
        ['\alpha= xxx; \beta=', mat2str(paramsGen.beta),'; phases: ', trainingstring]};
elseif size(paramsRW.alpha,2) <= 7 & length(paramsGen.beta) > 7
    titlestring= {['unique cues:' num2str(paramsRW.UC), ',  modified salience:', num2str(paramsRW.mUC),';'];...
        ['\alpha=', mat2str(paramsRW.alpha'),' \beta=xxx; phases: ', trainingstring]};
else
    titlestring= {['unique cues:' num2str(paramsRW.UC), ',  modified salience:', num2str(paramsRW.mUC),';'];...
        ['\alpha= xxx; \beta=xxx ; phases: ', trainingstring]};
end

if paramsRW.weightPlot
    plotWeigthCourse(weights(:,:,ss+1),'RW: Associative weights of elements ', titlestring);
end
if paramsRW.activationPlots
    plotOutputActivation(TestAktivierung(:,:,ss+1),paramsGen.phaseInfo,'RW: Hypothetical US activations by CSs of different trial types', titlestring,paramsGen.Stimulusname );
end
if paramsRW.trainedPlot
    plotAmatrix(amatrix(:,:,ss+1),paramsGen.phaseInfo,'RW: US activations by CSs of current trial', titlestring, [paramsGen.Stimulusname]);
end

%Speichern der Gewichtsmatrix
speicherpath=paramsRW.resultsPath;
if ~isempty(speicherpath{2});
    if isempty(speicherpath{1}) %if not flder is specified, use current folder
        speicherpath{1}=pwd;
    end
    dlmwrite (fullfile(speicherpath{1},[speicherpath{2}, '_testactivation.txt']), [TestAktivierung(:,:,ss+1), paramsGen.phaseInfo], '\t') ;
    dlmwrite (fullfile(speicherpath{1},[speicherpath{2}, '_trainedactivation.txt']), [amatrix(:,:,ss+1), paramsGen.phaseInfo], '\t')
    dlmwrite (fullfile(speicherpath{1},[speicherpath{2}, '_weights.txt']),weights(:,:, ss+1),'\t') ;
end




%%% SUBFUNCTIONS

function cs_code = expandMatrix_UC(cs)
%function that expands the CSmatrix computing all combinations of elements
%output is matrix with unique cue code for stimuli
idx = 1:size(cs,2); %number of elements

for i=1:size(cs,1); %number of trials / CS types
    cs_matrix =    cs(i,:); % CS_matrix with elemental units
    for j=2:size(cs,2) % combination of elements
        comb = nchoosek (idx, j);
        dummy=1;
        for k=1:size (comb,2);
            dummy = (dummy .* cs_matrix (comb(:,k)));
        end
        cs_matrix = [cs_matrix dummy];
    end
    cs_code(i,:)=cs_matrix;
end
%-------------------------------------------------------------------------------
%     This file is part of ALTSim.
% 
%     ALTSim is free software: you can redistribute it and/or modify
%     it under the terms of the GNU General Public License as published by
%     the Free Software Foundation, either version 3 of the License, or
%     (at your option) any later version.
% 
%     ALTSim is distributed in the hope that it will be useful,
%     but WITHOUT ANY WARRANTY; without even the implied warranty of
%     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%     GNU General Public License for more details.
% 
%     You should have received a copy of the GNU General Public License
%     along with ALTSim.  If not, see <http://www.gnu.org/licenses/>.